import argparse
import re
import time
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
from selenium.common.exceptions import NoSuchElementException

class BJWebScraper:
    def __init__(self, url, begin_page, end_page, id, output_file = "output_file.csv", verbose=False):
        self.url = url
        self.begin_page = begin_page
        self.end_page = end_page
        self.id = id
        self.output_file = output_file
        self.verbose = verbose
        self.browser = None

    def is_ready(self, browser):
        return browser.execute_script(r"""
            return document.readyState === 'complete'
        """)

    def fetch_total_record(self):
        options = webdriver.ChromeOptions()
        if self.verbose:
            pass
        else:
            options.add_argument("headless")
        browser = webdriver.Chrome(options=options)
        # Load web page
        browser.get(self.url)
        # Network transport takes time. Wait until the page is fully loaded
        WebDriverWait(browser, 60).until(self.is_ready)
        el = browser.find_element(By.ID, 'filter_EQUAL_SBLX')
        for option in el.find_elements(By.TAG_NAME,'option'):
            if option.text == '塔式起重机':
                option.click() # select() in earlier versions of webdriver
                break
        # simulate click on browser,query with above filter
        query_img_btn = browser.find_element(By.XPATH, "//input[@type='image']")
        query_img_btn.click()
        WebDriverWait(browser, 60).until(self.is_ready)
        
        #fetch total record using re
        text = browser.page_source

        match = re.search(r'总记录数:(\d+)', text)
        if match:
            total_records = match.group(1)
            print(total_records)
            return total_records
        else:
            print("Match not found")
            return 0

    def scrape_onepage(self, page):
        # option, todo
        ret = 0
        #options = webdriver.ChromeOptions()
        ret = self.init_browser()
        if ret != 0:
            print(f"page {page}: fail to init_browser")
            return ret
        for i in range(10):
            ret = self.check_captcha()
            if ret != 0:
                print(f"page {page}: fail to check_captcha")
                if i < 9:
                    self.browser.refresh()
                    continue
                else:
                    return ret
            else:
                break
        ret = self.expand_query_button()
        if ret != 0:
            print(f"page {page}: fail to expand_query_button")
            return ret
        ret = self.jump_to_page(page)
        if ret != 0:
            print(f"page {page}: fail to jump_to_page")
            return ret
        ret = self.extract_data_to_file()
        if ret != 0:
            print(f"page {page}: fail to extract_data_to_file")
        self.uninit_browser()

    def scrape_onepage_old(self, page):
        # scrape data and save to output_file
        #updated = pyqtSignal(str)
        options = webdriver.ChromeOptions()
        if self.verbose:
            pass
        else:
            options.add_argument("headless")
        browser = webdriver.Chrome(options=options)
        # Load web page
        browser.get(self.url)
        time.sleep(2)
        # Network transport takes time. Wait until the page is fully loaded
        WebDriverWait(browser, 60).until(self.is_ready)
        el = browser.find_element(By.ID, 'filter_EQUAL_SBLX')
        for option in el.find_elements(By.TAG_NAME,'option'):
            if option.text == '塔式起重机':
                option.click() # select() in earlier versions of webdriver
                break
        # simulate click on browser,query with above filter
        query_img_btn = browser.find_element(By.XPATH, "//input[@type='image']")
        query_img_btn.click()
        time.sleep(2)
        WebDriverWait(browser, 60).until(self.is_ready)
        f = open(self.output_file, "a")
        #find page edit
        #print(f'debug jumpPageBox element...{browser.page_source}')
        input_page_element = browser.find_element(By.XPATH, "//input[@id='jumpPageBox']")
        #find jump button
        jump_button_element = browser.find_element(By.XPATH, "//input[@name='跳转']")
        input_page_element.clear()
        time.sleep(2)
        print(f"debug.page jumping... {int(page)}")
        input_page_element.send_keys(int(page))
        time.sleep(2)
        #print(f'debug jump button ... {browser.page_source}')
        jump_button_element.click()
        time.sleep(2)
        print(f'jump to page {int(page)}...')
        WebDriverWait(browser, 60).until(self.is_ready)
        # grab data from search result in table
        elements = browser.find_elements(By.XPATH, "//table[@class='gridview']")
        for elem in elements:
            if self.verbose:
                print(elem.text)
            f.write(elem.text)
        f.close()

        browser.close()

    def scrape(self):
        pass
    
    def init_browser(self, options=webdriver.ChromeOptions()):
        if self.verbose:
            pass
        else:
            options.add_argument("headless")
             
        if self.browser is None:
            browser = webdriver.Chrome(options=options)
            self.browser = browser
        if self.browser is not None:
            self.browser.get(self.url)
            WebDriverWait(self.browser, 30).until(self.is_ready)
            return 0
        else:
            return -1   # fail to init browser

    def uninit_browser(self):
        self.browser.close()
        self.browser = None
        return 0
    
    def check_captcha(self, option=False):
        if option == False:
            return 0
        # # make sure captcha solved correctly
        # if self.browser is None:
        #     print("browser not init")
        #     return -1
        # self.browser.get(self.url)
        # #self.browser.refresh()
        # #self.browser.maximize_window()
        # # Network transport takes time. Wait until the page is fully loaded
        # WebDriverWait(self.browser, 30).until(self.is_ready)
        
        return 0

    def expand_query_button(self):
        if self.browser is None:
            print("browser not init")
            return -1
        if self.verbose:
            print("find query and click")
        try:
            el = self.browser.find_element(By.ID, 'filter_EQUAL_SBLX')
            for option in el.find_elements(By.TAG_NAME,'option'):
                if option.text == '塔式起重机':
                    option.click() # select() in earlier versions of webdriver
                    break
            time.sleep(1)
            # simulate click on browser,query with above filter
            query_img_btn = self.browser.find_element(By.XPATH, "//input[@type='image']")
            query_img_btn.click()
            time.sleep(2)
            WebDriverWait(self.browser, 60).until(self.is_ready)

            return 0
        except NoSuchElementException:
            print("exception in expand_query_button")
            return -2
        except Exception as e:
            print(f'{e} rasied in expand_query_button')
            return -3

    def jump_to_page(self, page):
        if self.browser is None:
            print("browser not init")
            return -1
        try:
            #find page edit
            #print(f'debug jumpPageBox element...{browser.page_source}')
            #fetch total record using re
            text = self.browser.page_source
            total_records = 0
            match = re.search(r'总记录数:(\d+)', text)
            if match:
                total_records = match.group(1)
            else:
                total_records = 0
            if total_records == 0:
                return -1 # no record

            input_page_element = self.browser.find_element(By.XPATH, "//input[@id='jumpPageBox']")
            #find jump button
            jump_button_element = self.browser.find_element(By.XPATH, "//input[@name='跳转']")
            input_page_element.clear()
            time.sleep(1)
            input_page_element.send_keys(int(page))
            time.sleep(1)
            jump_button_element.click()
            time.sleep(2)
            print(f'jump to page {int(page)}...')
            WebDriverWait(self.browser, 60).until(self.is_ready)
            return 0
        except NoSuchElementException:
            print("exception in jump_to_page")
            return -2
        except Exception as e:
            print(f'{e} rasied in jump_to_page')
            return -3

    def extract_data_to_file(self):
        if self.browser is None:
            print("browser not init")
            return -1
        try:
            f = open(self.output_file, "a")
            # grab data from search result in table
            elements = self.browser.find_elements(By.XPATH, "//table[@class='gridview']")
            for elem in elements:
                if self.verbose:
                    print(elem.text)
                f.write(elem.text)
            f.close()
            return 0
        except NoSuchElementException:
            print("exception in extract_data_to_file")
            return -2
        except Exception as e:
            print(f'{e} raised in extract_data_to_file')
            return -3


# # Example usage:
# parser = argparse.ArgumentParser(description='Grab one page each time according to argument passed')
# parser.add_argument('-b', '--beginpage', help='begin page number specified', required=True)
# parser.add_argument('-e', '--endpage', help='end page number specified', required=True)
# parser.add_argument('-f', '--outputfile', help='output file to save grabbed data', required=True)
# parser.add_argument('-v', '--verbose', help='Increase output verbosity', action='store_true')

# args = parser.parse_args()

# begin_page = int(args.beginpage)
# end_page = int(args.endpage)
# output_file = args.outputfile

# output_file = output_file + '-' + f'{begin_page}' + '-to-' + f'{end_page}' + '.csv'
# verbose = False;
# if args.verbose:
#     print('Verbose output enabled')
#     verbose = True
# print(f"handle page number: {begin_page} to {end_page}")
# scraper = BJWebScraper(
#     'http://bjjs.zjw.beijing.gov.cn/eportal/ui?pageId=308903', 
#     begin_page, end_page, output_file, verbose)
# scraper.scrape()
