import os
import sys
import math
import datetime
from PyQt6.QtCore import Qt, QThread, pyqtSignal, QMetaType, QCoreApplication
from PyQt6.QtWidgets import QApplication, QMainWindow, QTextEdit, QLayout, QFrame, QSpinBox, QDialog, QMenu
from PyQt6.QtWidgets import QApplication, QLabel, QComboBox, QHBoxLayout, QVBoxLayout, QWidget, QLineEdit, QPushButton, QFileDialog, QTextEdit, QProgressBar, QCheckBox
from PyQt6.QtGui import QAction
from PyQt6.QtGui import QIcon, QPixmap
from PyQt6.QtGui import QTextCursor
from PyQt6.QtGui import QIntValidator
from BJWebScraper import BJWebScraper
from TJWebScraper import TJWebScraper
import threading

class WorkerThread0(QThread):
    # 用于向主线程发送信号的自定义信号
    total_record_updated = pyqtSignal(int)
    def __init__(self, scraper):
        super().__init__()
        self.scraper = scraper

    def run(self):
        #print("thread run...")
        total_records = self.scraper.fetch_total_record()
        self.total_record_updated.emit(int(total_records))


class WorkerThread(QThread):
    # 用于向主线程发送信号的自定义信号
    progress_updated = pyqtSignal(int)
    def __init__(self, scraper):
        super().__init__()
        self.scraper = scraper

    def run(self):
        exception_raised = False
        current_page = 0
        print(f'run in thread, page {self.scraper.begin_page} to {self.scraper.end_page} ')
        try:
            # option, todo
            ret = 0
            #options = webdriver.ChromeOptions()
            ret = self.scraper.init_browser()
            if ret != 0:
                print(f"gui fail to init_browser")
                return
            for ii in range(10):
                ret = self.scraper.check_captcha()
                if ret != 0:
                    print(f"{ii} try, gui fail to check_captcha")
                    if ii < 9:
                        self.scraper.browser.refresh()
                        continue
                else:
                    break
            ret = self.scraper.expand_query_button()
            if ret != 0:
                print(f"gui fail to expand_query_button, ret: {ret}")
                return
            for page in range(self.scraper.begin_page, self.scraper.end_page + 1):
                current_page = page
                for jj in range(10):
                    ret = self.scraper.jump_to_page(page)
                    if ret != 0:
                        print(f"{jj} try, gui page {page}: fail to jump_to_page, ret: {ret}")
                        if jj < 2:
                            continue
                        else:
                            return
                    else:
                        break
                for jj in range(3):
                    ret = self.scraper.extract_data_to_file()
                    if ret != 0:
                        print(f"gui page {page}: fail to extract_data_to_file, ret: {ret}")
                        if jj < 2:
                            continue
                        else:
                            return
                    else:
                        break
                self.progress_updated.emit(int(page))
            self.scraper.uninit_browser()
        except Exception as e:
            exception_raised = True
            print(f'gui exception <<< {e} >>> raised during handle page : {current_page}')
        finally:
            if exception_raised:
                self.progress_updated.emit(-1)



class AboutDialog(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        
        self.setWindowTitle("About 数据搬运工")
        
        about_label = QLabel("数据搬运工 version 1.0.0")
        about_layout = QVBoxLayout()
        about_layout.addWidget(about_label)
        
        self.setLayout(about_layout)


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.total_records = 0
        self.begin_page = 1
        self.total_pages = 0
        self.end_page = 1
        self.batch_factor = 100
        #init UI data if necessary
        self.url_prefix = {
            0: "bj_",
            1: "tj_"
        }
        self.previous_url_index = 0
        self.verbose = False
        self.record_count_per_page = 15
        self.start_time = datetime.datetime.now()
        self.stop_time = datetime.datetime.now()
        self.enable_multithread = True
        self.pages_per_thread = 50
        self.worker_thread_array = []
        self.progress_bar_page_count = 0
        self.file_number = 1
        self.pages_pair_array = []
        self.scraper_array = []
        self.initUI()
        self.thread_lock = threading.Lock()

    def on_site_combo_selection_change(self, index):
        # Handle selection change event here
        url_index = self.site_combo.currentIndex()
        if url_index != self.previous_url_index:
            self.previous_url_index = url_index
            self.type_combo.clear()
            if url_index == 0:
                self.record_count_per_page = 15
                self.type_combo.addItems(["塔式起重机", "物料提升机", "施工升降机"])
            elif url_index == 1:
                self.record_count_per_page = 10
                self.type_combo.addItems(["塔式起重机", "施工升降机", "物料提升机", "桩工机械", "高处作业吊篮", "附着升降式脚手架", "盾构机", "吕带式起重机", "门(桥)式起重机", "架桥机", "货用施工升降机", "整体提升模板"])

    def on_verbose_checkbox_state_change(self, state):
        if state == 2:  # 2 = Checked
            self.verbose = True
        else:
            self.verbose = False
    def on_multithread_checkbox_state_change(self, state):
        if state == 2:  # 2 = Checked
            self.enable_multithread = True
        else:
            self.enable_multithread = False

    def log_edit_scroll_bar_moved(self, value):
        # 如果垂直滚动条滑块位置在最大值的位置，就让TextEdit自动滚动
        if value == self.log_edit.verticalScrollBar().maximum():
            cursor = self.log_edit.textCursor()
            cursor.movePosition(QTextCursor.MoveOperation.End)
            self.log_edit.setTextCursor(cursor)

    def show_about_dialog(self):
        about_dialog = AboutDialog(self)
        about_dialog.exec()

    def initUI(self):
        icon = QIcon("icons/app_icon.png")
        self.setWindowIcon(icon)
        self.setWindowTitle("数据搬运工")
        self.window = QWidget(self)
        # 网站选择下拉菜单
        self.site_label = QLabel("选择需要抓取的网站:")
        self.site_combo = QComboBox(self)
        self.site_combo.addItems(["http://bjjs.zjw.beijing.gov.cn/eportal/ui?pageId=308903", "http://60.29.202.86:8090/#/ViewBsdtJxbaList"])
        # 设备类型选择下拉菜单
        self.type_label = QLabel("选择设备类型:")
        self.type_combo = QComboBox()
        self.type_combo.addItems(["塔式起重机", "物料提升机", "施工升降机"])
        self.site_combo.currentIndexChanged.connect(self.on_site_combo_selection_change)
        # 查询网站内容的编辑框和按钮
        self.query_label = QLabel("查询网站数据记录总数:")
        self.query_edit = QLineEdit()
        self.total_page_label = QLabel("页码总数：")
        self.total_page_label.setStyleSheet("color: red")
        separator = QFrame()
        separator.setFrameShape(QFrame.Shape.VLine)  # Set the frame shape to a vertical line
        separator.setFrameStyle(QFrame.Shape.Panel | QFrame.Shadow.Sunken)
        self.query_button = QPushButton()
        self.query_button.setIcon(QIcon(QPixmap("icons/query_icon.png")))  # 设置查询按钮图标
        self.query_button.setToolTip("点击按钮，查询需要下载的记录总数...")
        self.begin_page_label = QLabel("开始页码")
        self.begin_page_edit = QLineEdit()
        self.end_page_label = QLabel("结束页码")
        self.end_page_edit = QLineEdit()
        # 设置查询按钮样式为图片按钮
        self.query_button.setStyleSheet("""
            QPushButton {
                border-image: url(icons/query_button.png) 0 0 0 0 stretch stretch;
                padding: 0;
                min-width: 0;
            }
        """)
        # 查询网站内容按钮点击事件
        self.query_button.clicked.connect(self.start_worker_thread0)
        # 保存文件路径编辑框和按钮
        self.path_label = QLabel("保存目录路径:")
        self.path_edit = QLineEdit()
        self.path_button = QPushButton()
        self.path_button.setIcon(QIcon(QPixmap("icons/folder_icon.png")))  # 设置打开文件夹图标
        self.path_button.setToolTip("点击按钮，选择保存数据的文件根目录")
        # 设置打开文件夹按钮样式为图片按钮
        self.path_button.setStyleSheet("""
            QPushButton {
                border-image: url(icons/folder_button.png) 0 0 0 0 stretch stretch;
                padding: 0;
                min-width: 0;
            }
        """)
        #self.path_button.clicked.connect(self.open_path_dialog)
        self.path_button.clicked.connect(self.open_data_path)
        self.filepath_label = QLabel("生成文件名路径:")
        self.filepath_edit = QLineEdit()
        self.multithread_checkbox = QCheckBox('multithread', self)
        self.multithread_checkbox.stateChanged.connect(self.on_multithread_checkbox_state_change)
        self.multithread_checkbox.setToolTip("Enable multirhread to grab data")
        self.multithread_checkbox.setChecked(self.enable_multithread)
        self.pages_per_thread_spin_box_factor = QSpinBox()
        self.pages_per_thread_spin_box_factor.setRange(10, 50)
        self.pages_per_thread_spin_box_factor.setToolTip("每个线程抓取的页面数")
        self.pages_per_thread_spin_box_factor.setValue(self.pages_per_thread)
        self.verbose_checkbox = QCheckBox('verbose', self)
        self.verbose_checkbox.stateChanged.connect(self.on_verbose_checkbox_state_change)
        self.verbose_checkbox.setToolTip("Debug Mode, Popup Browser Window")
        # 显示网站内容的标签和log的文本编辑框
        self.content_label = QLabel(self.window)
        self.log_label = QLabel("程序运行输出日志:")
        self.log_edit = QTextEdit(self.window)
        self.log_edit.verticalScrollBar().valueChanged.connect(self.log_edit_scroll_bar_moved)
        # 添加到布局管理器中
        self.mylayout = QVBoxLayout()
        self.window.hlayout1 = QHBoxLayout()
        self.window.hlayout1.addWidget(self.site_label)
        self.window.hlayout1.addWidget(self.site_combo)
        self.window.hlayout1.addWidget(self.type_label)
        self.window.hlayout1.addWidget(self.type_combo)
        self.mylayout.addLayout(self.window.hlayout1)
        self.window.hlayout2 = QHBoxLayout()
        self.window.hlayout2.addWidget(self.query_label)
        self.window.hlayout2.addWidget(self.query_edit)
        self.window.hlayout2.addWidget(self.total_page_label)
        self.window.hlayout2.addWidget(separator)
        self.window.hlayout2.addWidget(self.begin_page_label)
        self.window.hlayout2.addWidget(self.begin_page_edit)
        self.window.hlayout2.addWidget(self.end_page_label)
        self.window.hlayout2.addWidget(self.end_page_edit)
        self.begin_page_edit.setToolTip("指定抓取起始页码， 推荐做法")
        self.end_page_edit.setToolTip("指定抓取结束页码，推荐做法")
        int_validator = QIntValidator()
        self.query_edit.setValidator(int_validator)
        self.begin_page_edit.setValidator(int_validator)
        self.end_page_edit.setValidator(int_validator)
        self.window.hlayout2.addWidget(self.query_button)
        self.mylayout.addLayout(self.window.hlayout2)
        self.window.hlayout3 = QHBoxLayout()
        self.window.hlayout3.addWidget(self.path_label)
        self.window.hlayout3.addWidget(self.path_edit)
        self.window.hlayout3.addWidget(self.path_button)
        self.mylayout.addLayout(self.window.hlayout3)
        self.window.hlayout31 = QHBoxLayout()
        self.window.hlayout31.addWidget(self.filepath_label)
        self.window.hlayout31.addWidget(self.filepath_edit)
        self.window.hlayout31.addWidget(self.multithread_checkbox)
        self.window.hlayout31.addWidget(self.pages_per_thread_spin_box_factor)
        self.window.hlayout31.addWidget(self.verbose_checkbox)
        self.mylayout.addLayout(self.window.hlayout31)
        self.window.hlayout4 = QHBoxLayout()
        self.progress_bar = QProgressBar()
        self.progress_bar.setMaximum(30000)
        self.execute_button = QPushButton()
        self.execute_button.setIcon(QIcon(QPixmap("icons/execute_icon.png")))  # 设置查询按钮图标
        self.execute_button.clicked.connect(self.start_worker_multithread)
        self.execute_button.setToolTip("点击按钮，下载指定页面数据记录")
        self.spin_box_factor = QSpinBox()
        self.spin_box_factor.setRange(1, 10)
        self.spin_box_factor.setValue(int(self.batch_factor/10))
        #self.spin_box_factor.setReadOnly(True)
        #self.spin_box_factor.setButtonSymbols(QSpinBox.UpDownArrows)
        self.nextbatch_button = QPushButton()
        self.nextbatch_button.setIcon(QIcon(QPixmap("icons/next_icon.png")))  # 设置查询按钮图标
        self.nextbatch_button.clicked.connect(self.get_nextbatch_pages)
        self.nextbatch_button.setToolTip(f"点击按钮，选择下一组页面,{self.batch_factor}个页面一组")
        self.window.hlayout4.addWidget(self.progress_bar)
        self.window.hlayout4.addWidget(self.spin_box_factor)
        self.window.hlayout4.addWidget(self.execute_button)
        self.window.hlayout4.addWidget(self.nextbatch_button)
        self.mylayout.addLayout(self.window.hlayout4)
        self.window.hlayout5 = QVBoxLayout()
        self.window.hlayout5.addWidget(self.content_label)
        self.window.hlayout5.addWidget(self.log_label)
        self.window.hlayout5.addWidget(self.log_edit)
        self.mylayout.addLayout(self.window.hlayout5)
        self.window.setLayout(self.mylayout)
        self.setCentralWidget(self.window)
        self.begin_page_edit.setText(str(self.begin_page))
        self.end_page_edit.setText(str(self.begin_page+self.batch_factor-1))
        #sys.stdout = self
        #sys.stderr = self
        #print('Hello, world!')
        #current_dir = QCoreApplication.applicationDirPath()
        current_dir = os.path.expanduser("~")
        default_data_dir = current_dir + '/data'
        # 检查目录是否存在
        if not os.path.exists(default_data_dir):
            # 如果目录不存在，则创建它
            os.makedirs(default_data_dir)
            print("已创建目录：", default_data_dir)
        else:
            print("目录已存在：", default_data_dir)
        self.path_edit.setText(default_data_dir)
        # 将微调框的 valueChanged 信号连接到更新标签的函数上
        self.spin_box_factor.valueChanged.connect(self.update_batch_factor)
        self.pages_per_thread_spin_box_factor.valueChanged.connect(self.update_pages_per_thread_factor)
        # create File menu
        file_menu = QMenu("File", self)
        self.menuBar().addMenu(file_menu)
        
        # create About action
        about_action = QAction("关于数据搬运工", self)
        about_action.triggered.connect(self.show_about_dialog)
        file_menu.addAction(about_action)

    # 定义一个函数，在微调框值发生变化时更新变量，每个线程抓取的页面数
    def update_pages_per_thread_factor(self, value):
       self.pages_per_thread = int(value)
 
    # 定义一个函数，在微调框值发生变化时更新标签文本
    def update_batch_factor(self, value):
        self.batch_factor = 10*int(value) # const base factor: 10
        self.nextbatch_button.setToolTip(f"点击按钮，选择下一组页面,{self.batch_factor}个页面一组")
        begin_page = int(self.begin_page_edit.text())
        end_page = begin_page + self.batch_factor - 1
        self.end_page_edit.setText(str(end_page))

    def write(self, text):
        self.log_edit.insertPlainText(text + '\n\r')

    def main_logout(self, text):
        now = datetime.datetime.now()
        now_str = now.strftime("%Y-%m-%d %H:%M:%S")
        self.log_edit.append(f'[{now_str}] {text}')

    def open_data_path(self):
        current_dir = QCoreApplication.applicationDirPath()
        current_dir = os.path.expanduser("~")  # fixme
        default_data_dir = current_dir + '/data'
        # 检查目录是否存在
        if not os.path.exists(default_data_dir):
            # 如果目录不存在，则创建它
            os.makedirs(default_data_dir)
            print("已创建目录：", default_data_dir)
        else:
            print("目录已存在：", default_data_dir)
        folder_path = QFileDialog.getExistingDirectory(None, "选择目录", default_data_dir)
        if folder_path:
            self.path_edit.setText(folder_path)

    # 保存文件路径按钮点击事件
    def open_path_dialog(self):
        path = QFileDialog.getSaveFileName(self, "选择保存文件路径", ".", "CSV files (*.csv)")
        if path:
            self.path_edit.setText(path[0])
    
    def get_nextbatch_pages(self):
        self.begin_page = self.begin_page + self.batch_factor
        self.end_page = self.begin_page + self.batch_factor - 1
        self.begin_page_edit.setText(str(self.begin_page))
        self.end_page_edit.setText(str(self.end_page))

    def start_worker_multithread(self):
        # 创建线程对象并连接信号
        # 在这里写查询网站内容的代码
        if self.total_records == 0 and len(self.query_edit.text()) == 0:
            self.main_logout("=====总记录数是0,请先点击查询按钮=====")
            return
        try:
            self.total_records = int(self.query_edit.text())
        except:
            self.main_logout("=====请输入正确的总记录数,或者请先点击查询按钮=====")
            return
        self.start_time = datetime.datetime.now()
        url = self.site_combo.currentText()
        url_index = self.site_combo.currentIndex()
        print(f'site combo index: {url_index}')
        if url_index == 0:
            self.record_count_per_page = 15
        elif url_index == 1:
            self.record_count_per_page = 10

        self.main_logout(f"=====开始抓取网站数据，网站：{url}")
        if len(self.begin_page_edit.text()) == 0:
            print("begin_page_edit empty?")
            self.begin_page = 1
        else:
            self.begin_page = int(self.begin_page_edit.text())
        if len(self.end_page_edit.text()) == 0:
            print("end_page_edit empty?")
            self.end_page = math.ceil(self.total_records / record_count_per_page)
        else:
            self.end_page = int(self.end_page_edit.text())
        if self.begin_page <=0:
            self.begin_page = 1
        if self.end_page > self.total_pages:
            self.end_page = self.total_pages
        print(f'=====begin_page: {self.begin_page},end_page: {self.end_page}=====')
        
        self.output_file = self.path_edit.text() + "/" + self.url_prefix[url_index]  +  f'page_{self.begin_page}-to-{self.end_page}' + '_parts'
        self.filepath_edit.setText(self.output_file)
        #scraper.scrape()
        total_pages = self.end_page - self.begin_page + 1
        self.progress_bar.setRange(0, total_pages)
        self.progress_bar.setValue(0)
        self.progress_bar_page_count = 0
        print(f'progress bar max: {self.progress_bar.maximum()}')
        if self.enable_multithread:
            thread_number = math.ceil(total_pages / self.pages_per_thread)
            self.file_number = thread_number # generate files
            print(f'thread number: {thread_number}')
            # each starge page per thread?
            self.pages_pair_array.clear()
            self.scraper_array.clear()
            self.worker_thread_array.clear()
            for thread_i in range(thread_number):
                begin_page = self.begin_page + thread_i*self.pages_per_thread
                end_page = begin_page + self.pages_per_thread - 1
                if end_page > self.end_page:
                    end_page = self.end_page
                    self.pages_pair_array.append((begin_page, self.end_page))
                self.pages_pair_array.append((begin_page, end_page))
                print(f'thread {thread_i},begin_page: {begin_page}, end_page: {end_page}')
                debug_msg = f'thread {thread_i} handle page {self.pages_pair_array[thread_i][0]} to {self.pages_pair_array[thread_i][1]}'
                print(f'{debug_msg}')
                self.main_logout(f'===== {debug_msg}')
                output_file = self.output_file + f'_{thread_i}.csv'
                print(f'input page: {self.pages_pair_array[thread_i][0]}, {self.pages_pair_array[thread_i][1]}')
                # init each thread
                if url_index == 0:
                    scraper = BJWebScraper(
                        url,
                        self.pages_pair_array[thread_i][0], self.pages_pair_array[thread_i][1], url_index, output_file, self.verbose) 
                elif url_index == 1:
                    scraper = TJWebScraper(
                        url,
                        self.pages_pair_array[thread_i][0], self.pages_pair_array[thread_i][1], url_index, output_file, self.verbose) 
                self.scraper_array.append(scraper)
                self.worker_thread_array.append(WorkerThread(self.scraper_array[thread_i]))
                self.worker_thread_array[thread_i].progress_updated.connect(self.update_progress_bar)
                # 启动线程
                self.worker_thread_array[thread_i].start()
                self.execute_button.setEnabled(False)
                self.nextbatch_button.setEnabled(False)
        else:
            #scraper.scrape()
            self.progress_bar.setRange(self.begin_page-1, self.end_page)
            self.progress_bar.setValue(self.begin_page-1)
            if url_index == 0:
                scraper = BJWebScraper(
                        url,
                        self.begin_page, self.end_page, url_index, self.output_file, self.verbose) 
            elif url_index == 1:
                scraper = TJWebScraper(
                        url,
                        self.begin_page, self.end_page,url_index, self.output_file, self.verbose)
            self.worker_thread = WorkerThread(scraper)
            self.worker_thread.progress_updated.connect(self.update_progress_bar)
            # 启动线程
            self.worker_thread.start()
            self.execute_button.setEnabled(False)
            self.nextbatch_button.setEnabled(False)

    def combine_all_part_file(self, count):
        final_filename = self.output_file + '.csv'
        self.main_logout(f"所有页面抓取成功，开始合并生成文件... {final_filename}")
        #final_handle = open(final_filename, "a")
        file_part_prefix = self.output_file
        with open(final_filename, 'w') as output_file:
            for i in range(count):
                file_part_file = file_part_prefix + f'_{i}.csv'
                self.main_logout(f'===读取文件{file_part_file}')
                with open(file_part_file, 'r') as input_file:
                    for line in input_file:
                        output_file.write(line)
        self.main_logout(f"完成合并，生成文件... {final_filename}")

    def start_worker_thread(self):
        # 创建线程对象并连接信号
        # 在这里写查询网站内容的代码
        if self.total_records == 0 and len(self.query_edit.text()) == 0:
            self.main_logout("=====总记录数是0,请先点击查询按钮=====")
            return
        try:
            self.total_records = int(self.query_edit.text())
        except:
            self.main_logout("=====请输入正确的总记录数,或者请先点击查询按钮=====")
            return
        self.start_time = datetime.datetime.now()
        url = self.site_combo.currentText()
        url_index = self.site_combo.currentIndex()
        print(f'site combo index: {url_index}')
        if url_index == 0:
            self.record_count_per_page = 15
        elif url_index == 1:
            self.record_count_per_page = 10

        self.main_logout(f"=====开始抓取网站数据，网站：{url}")
        if len(self.begin_page_edit.text()) == 0:
            print("begin_page_edit empty?")
            self.begin_page = 1
        else:
            self.begin_page = int(self.begin_page_edit.text())
        if len(self.end_page_edit.text()) == 0:
            print("end_page_edit empty?")
            self.end_page = math.ceil(self.total_records / record_count_per_page)
        else:
            self.end_page = int(self.end_page_edit.text())
        if self.begin_page <=0:
            self.begin_page = 0
        if self.end_page > self.total_pages:
            self.end_page = self.total_pages
        print(f'=====begin_page: {self.begin_page},end_page: {self.end_page}=====')
        
        self.output_file = self.path_edit.text() + "/" + self.url_prefix[url_index]  +  f'page_{self.begin_page}-to-{self.end_page}' + '.csv'
        self.filepath_edit.setText(self.output_file)
        if url_index == 0:
            scraper = BJWebScraper(
                        url,
                        self.begin_page, self.end_page, url_index, self.output_file, self.verbose) 
        elif url_index == 1:
            scraper = TJWebScraper(
                        url,
                        self.begin_page, self.end_page,url_index, self.output_file, self.verbose) 
        #scraper.scrape()
        self.progress_bar.setRange(self.begin_page-1, self.end_page)
        self.progress_bar.setValue(self.begin_page-1)
        self.worker_thread = WorkerThread(scraper)
        self.worker_thread.progress_updated.connect(self.update_progress_bar)
        # 启动线程
        self.worker_thread.start()
        self.execute_button.setEnabled(False)
        self.nextbatch_button.setEnabled(False)

    def start_worker_thread0(self):
        # 创建线程对象并连接信号,查询记录总数
        url = self.site_combo.currentText()
        url_index = self.site_combo.currentIndex()
        print(f'url: {url}')
        self.main_logout(f"=====开始查询记录总数，网站：{url}")
        if url_index == 0:
            scraper = BJWebScraper(
                        url,
                        0, 0,url_index, "", self.verbose) # 随意，除了url，其他参数在这没有意义
        elif url_index == 1:
            scraper = TJWebScraper(
                        url,
                        0, 0,url_index, "", self.verbose) # 随意，除了url，其他参数在这没有意义

        self.worker_thread0 = WorkerThread0(scraper)
        self.worker_thread0.total_record_updated.connect(self.update_total_records)
        # 启动线程
        self.worker_thread0.start()

    def update_progress_bar(self, value):
        # 接收来自线程的信号并更新进度条
        if self.enable_multithread:
            self.main_logout(f'=====第 {value} 页抓取完成=====')
            #self.progress_bar.setValue(value)
            if value > 0:
                self.thread_lock.acquire()
                self.progress_bar_page_count = self.progress_bar_page_count + 1
                self.thread_lock.release()
                print(f'progress bar value: {self.progress_bar_page_count}')
                self.progress_bar.setValue(self.progress_bar_page_count)
            if self.progress_bar_page_count == self.progress_bar.maximum() or value < 0:
                self.execute_button.setEnabled(True)
                self.nextbatch_button.setEnabled(True)
                self.stop_time = datetime.datetime.now()
                time_cost = self.stop_time - self.start_time
                total_seconds = time_cost.total_seconds()
                average_cost_seconds = total_seconds / (self.end_page -self.begin_page + 1)
                self.main_logout(f'=====抓取 {self.begin_page} to {self.end_page} 页,耗时 {total_seconds:.3f} 秒, 平均: {average_cost_seconds:.3f}')
                #combine file
                if value > 0:
                    self.combine_all_part_file(self.file_number)
                else:
                    self.main_logout("出现一些页面抓取识别，放弃文件合并")
        else:
            self.main_logout(f'=====第 {value} 页抓取完成=====')
            if value > 0:
                self.progress_bar_page_count = self.progress_bar_page_count + 1
                self.progress_bar.setValue(self.progress_bar_page_count)
            if value == self.progress_bar.maximum() or value < 0:
                self.execute_button.setEnabled(True)
                self.nextbatch_button.setEnabled(True)
                self.stop_time = datetime.datetime.now()
                time_cost = self.stop_time - self.start_time
                total_seconds = time_cost.total_seconds()
                average_cost_seconds = total_seconds / (self.end_page -self.begin_page + 1)
                self.main_logout(f'=====抓取 {self.begin_page} to {self.end_page} 页,耗时 {total_seconds:.3f} 秒, 平均: {average_cost_seconds:.3f}')

    def update_total_records(self, value):
        self.total_records = int(value)
        self.total_pages = math.ceil(self.total_records/self.record_count_per_page)
        self.total_page_label.setText(f'页码总数：{self.total_pages}')
        self.query_edit.setText(f'{value}')
        self.main_logout(f"=====查询完成，记录总数: {value} 条=====")


if __name__ == '__main__':
    app = QApplication(sys.argv)
    main_window = MainWindow()
    main_window.show()
    sys.exit(app.exec())
