import os
import cv2
import numpy as np


class TJCaptchaHandler:
    def __init__(self, background_img, block_img, verbose=False):
        self.background_img = background_img
        self.block_img = block_img
        self.verbose = verbose
        self.target_contour = None
        self.source_contours = []
        self.source_x = 0
        self.target_x = 0
        
    def calculate_distance(self):
        image = cv2.imread(self.background_img)
        gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
        # Threshold the image to create a binary mask
        _, thresh = cv2.threshold(gray, 127, 255, cv2.THRESH_BINARY)
        # Find contours in the mask
        contours, hierarchy = cv2.findContours(thresh, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
        # find main puzzle contour in background
        if self.verbose:
            print(f"contour count: {len(contours)}")
        i=0
        for i in range(len(contours)):
            # Get the bounding rectangle of the contour
            if hierarchy[0][i][3] != -1:
                x, y, w, h = cv2.boundingRect(contours[i])
                aspect_ration = float(w/h)
                area = cv2.contourArea(contours[i])
                if area < 350:
                    continue
                if 0.8<= aspect_ration <=1.2:
                    self.source_contours.append(contours[i])
                    # Draw the rectangle on the original image
                    cv2.rectangle(image, (x, y), (x+w, y+h), (0, 255, 0), 2)
                    new_image = np.zeros((h, w, 3), dtype=np.uint8)
                    new_image[0:h, 0:w] = image[y:y+h, x:x+w]
                    # Display the output image
                    if self.verbose:
                        print(f'debug... {x}, {y}, {w}, {h}')
                        cv2.imshow(f"Output {i}", new_image)
                    i=i+1
        if self.verbose:
            cv2.imshow(f"original background", image)
            cv2.moveWindow("original background", 100, 100)
            pass
        # Load the image and convert it to grayscale
        image = cv2.imread(self.block_img)
        gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
        # Threshold the image to create a binary mask
        _, thresh = cv2.threshold(gray, 127, 255, cv2.THRESH_BINARY)
        # Find contours in the mask
        contours, hierarchy = cv2.findContours(thresh, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
        # Loop through all detected contours
        for contour in contours:
            # Get the bounding rectangle of the contour
            x, y, w, h = cv2.boundingRect(contour)
            # Calculate the aspect ratio of the rectangle
            aspect_ratio = float(w) / h
            # Check if the aspect ratio is within the acceptable range
            if 0.8 <= aspect_ratio <= 1.2:
                if self.verbose:
                    print(f"origina target contour: x, y, w, h = {x}, {y}, {w}, {h}")
                # Draw a green rectangle around the contour
                cv2.rectangle(image, (x, y), (x+w, y+h), (0, 255, 0), 2)
                new_image = np.zeros((h, w, 3), dtype=np.uint8)
                new_image[0:h, 0:w] = image[y:y+h, x:x+w]
                if self.verbose:
                    cv2.imshow("Output original", image)
                    cv2.moveWindow("Output original", 500, 100)
                    cv2.imshow("Output", new_image)
                self.target_contour = contour
        if self.verbose:
            print(f"source contour len: {len(self.source_contours)}")
        i = 0
        matched_index = 0
        matched_x = 0
        matched_xx = 0
        for cnt in self.source_contours:
            score = cv2.matchShapes(self.target_contour, cnt, cv2.CONTOURS_MATCH_I1, 0)
            if self.verbose:
                print(f" No. {i} contour matching score: {score}")
            x, y, w, h = cv2.boundingRect(self.target_contour)
            xx, yy, ww, hh = cv2.boundingRect(cnt)
            if self.verbose:
                print(f"x,  y,  w,  h :  {x}, {y}, {w}, {h}")
                print(f"xx, yy, ww, hh : {xx}, {yy}, {ww}, {hh}")
            if score < 0.5 and abs(yy-y) < 4:
                #matched_index = i
                matched_x = x
                matched_xx = xx
            i = i + 1    
        if self.verbose:
            cv2.waitKey(0)
            cv2.destroyAllWindows()
        return int(matched_xx-matched_x)


# if __name__== "__main__":
#     for i in range (0, 100):
#         print(f'===========Image {i}===============')
#         background_file = f'../captcha_image_data/dynamic_background_{i}.png'
#         block_file = f'../captcha_image_data/dynamic_block_{i}.png'
#         slider_captcha_handler = TJCaptchaHandler(background_file, block_file, True)

#         distance = slider_captcha_handler.calculate_distance()
#         print(f"distance: {distance}")


if __name__== "__main__":
    background_file = 'dynamic_background.png'
    block_file = 'dynamic_block.png'
    slider_captcha_handler = TJCaptchaHandler(background_file, block_file, True)
    distance = slider_captcha_handler.calculate_distance()
    print(f"distance: {distance}")

