import argparse
import os
import re
import time
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.common.exceptions import NoSuchElementException


from TJCaptchaHandler import TJCaptchaHandler
import base64


class TJWebScraper:
    def __init__(self, url, begin_page, end_page, id, output_file = "output_file.csv", verbose=False):
        self.url = url
        self.begin_page = begin_page
        self.end_page = end_page
        self.id = id
        self.output_file = output_file
        self.verbose = verbose
        self.captcha_handler = TJCaptchaHandler("dynamic_background.png", "dynamic_block.png")
        self.browser = None

    def is_ready(self, browser):
        return browser.execute_script(r"""
            return document.readyState === 'complete'
        """)

    def handle_captcha(self):
        # Launch Chrome browser in headless mode
        options = webdriver.ChromeOptions()
        if self.verbose:
            pass
        else:
            options.add_argument("headless")
        browser = webdriver.Chrome(options=options)
        # Load web page
        browser.get(self.url)
        # Network transport takes time. Wait until the page is fully loaded
        WebDriverWait(browser, 30).until(self.is_ready)
        #print(browser.page_source)
        #get captcha image verify-left-bar， verify-move-block
        div_element_moveblock = browser.find_element(By.CLASS_NAME, "verify-sub-block")
        actions = ActionChains(browser)
        actions.move_to_element(div_element_moveblock).perform()
        actions.release()
        time.sleep(1)
        # Find the div element with class "verify-img-panel"
        div_element_bgimg =  browser.find_element(By.CLASS_NAME, "verify-img-panel")
        #div_element = browser.find_element(By.XPATH, "//div[@class='verify-img-panel']")
        # Find the image element inside the div element
        image_element_bg = div_element_bgimg.find_element(By.TAG_NAME, "img")
        # Get the base64 encoded image data from the "src" attribute
        image_data_bg = image_element_bg.get_attribute("src")[22:]
        # Decode the base64 data to obtain the image binary data
        try:
            image_binary_bg = base64.b64decode(image_data_bg)
            # Save the image data to a file
            with open(f"dynamic_background.png", "wb") as f:
                f.write(image_binary_bg)
        except base64.binascii.Error as e:
            print("An error occurred while decoding background data:", str(e))
            return -1
        #find move block, important, take exact element for drag and move
        div_element_blockimg =  browser.find_element(By.CLASS_NAME, "verify-move-block")
        # Find the image element inside the div element
        image_element_block = div_element_blockimg.find_element(By.TAG_NAME, "img")
        # Get the base64 encoded image data from the "src" attribute
        image_data_block = image_element_block.get_attribute("src")[22:]
        # Decode the base64 data to obtain the image binary data
        try:
            image_binary_block = base64.b64decode(image_data_block)
            # Save the image data to a file
            with open(f"dynamic_block.png", "wb") as f:
                f.write(image_binary_block)
        except base64.binascii.Error as e:
            print("An error occurred while decoding slider data:", str(e))
            return -1
        if os.path.exists("dynamic_background.png") and os.path.exists("dynamic_block.png"):
            pass
        else:
            print("no captcha image data, please try again...")
            return -1
        distance = self.captcha_handler.calculate_distance()
        if self.verbose:
            print(f'distance: {distance}')
        if distance == 0:
            return -1
        distance = int(distance*1.277) # case by case
        actions.click_and_hold(div_element_blockimg).move_by_offset(distance+1, 0).release().perform()
        time.sleep(1)
        WebDriverWait(browser, 30).until(self.is_ready)
        browser.close()
        return 0
        
    def fetch_total_record(self):
        total_record_number = -1
        # Launch Chrome browser in headless mode
        options = webdriver.ChromeOptions()
        if self.verbose:
            pass
        else:
            options.add_argument("headless")
        browser = webdriver.Chrome(options=options)
        # Load web page
        browser.get(self.url)
        # Network transport takes time. Wait until the page is fully loaded
        WebDriverWait(browser, 30).until(self.is_ready)
        #print(browser.page_source)
        #get captcha image verify-left-bar， verify-move-block
        div_element_moveblock = browser.find_element(By.CLASS_NAME, "verify-sub-block")
        actions = ActionChains(browser)
        actions.move_to_element(div_element_moveblock).perform()
        actions.release()
        time.sleep(2)
        # Find the div element with class "verify-img-panel"
        div_element_bgimg =  browser.find_element(By.CLASS_NAME, "verify-img-panel")
        #div_element = browser.find_element(By.XPATH, "//div[@class='verify-img-panel']")
        # Find the image element inside the div element
        image_element_bg = div_element_bgimg.find_element(By.TAG_NAME, "img")
        # Get the base64 encoded image data from the "src" attribute
        image_data_bg = image_element_bg.get_attribute("src")[22:]
        # Decode the base64 data to obtain the image binary data
        try:
            image_binary_bg = base64.b64decode(image_data_bg)
            # Save the image data to a file
            with open(f"dynamic_background.png", "wb") as f:
                f.write(image_binary_bg)
        except base64.binascii.Error as e:
            if self.verbose:
                print("An error occurred while decoding background data:", str(e))
            return total_record_number
        #find move block
        div_element_blockimg =  browser.find_element(By.CLASS_NAME, "verify-move-block")
        # Find the image element inside the div element
        image_element_block = div_element_blockimg.find_element(By.TAG_NAME, "img")
        # Get the base64 encoded image data from the "src" attribute
        image_data_block = image_element_block.get_attribute("src")[22:]
        # Decode the base64 data to obtain the image binary data
        try:
            image_binary_block = base64.b64decode(image_data_block)
            # Save the image data to a file
            with open(f"dynamic_block.png", "wb") as f:
                f.write(image_binary_block)
        except base64.binascii.Error as e:
            if self.verbose:
                print("An error occurred while decoding slider data:", str(e))
            return total_record_number
        if os.path.exists("dynamic_background.png") and os.path.exists("dynamic_block.png"):
            pass
        else:
            print("no captcha image data, please try again...")
            return -1
        distance = self.captcha_handler.calculate_distance()
        print(f'distance: {distance}')
        if distance == 0:
            if self.verbose:
                print("distance is 0, might need to refresh capthca")
            return -1
        distance = int(distance*1.277)
        actions.click_and_hold(div_element_blockimg).move_by_offset(distance+1, 0).release().perform()
        time.sleep(2) # must have delay here
        WebDriverWait(browser, 30).until(self.is_ready)
        # find 展开 按钮
        expand_button_element = browser.find_element(By.XPATH, "//a[contains(text(), '展开')]")
        expand_button_element.click()
        WebDriverWait(browser, 30).until(self.is_ready)
        # select device type element
        device_type_label = browser.find_element(By.XPATH, "//label[text()='设备类别名称']")
        parent_of_device_type_label = device_type_label.find_element(By.XPATH, "..")
        grandparent_of_device_type_label = parent_of_device_type_label.find_element(By.XPATH, "..")
        device_type_combobox0 = grandparent_of_device_type_label.find_element(By.XPATH, ".//div[@class='ant-col ant-form-item-control-wrapper']") 
        #//////div[@class='ant-select ant-select-enabled']")
        device_type_combobox1 = device_type_combobox0.find_element(By.XPATH, ".//div[@class='ant-form-item-control']") 
        device_type_combobox2 = device_type_combobox1.find_element(By.XPATH, ".//span[@class='ant-form-item-children']") 
        device_type_combobox3 = device_type_combobox2.find_element(By.XPATH, ".//div[@class='ant-select ant-select-enabled']")
        device_type_combobox3.click()
        time.sleep(2)
        WebDriverWait(browser, 30).until(self.is_ready)
        device_type_combobox3.send_keys(Keys.ARROW_DOWN)
        device_type_combobox3.send_keys(Keys.ENTER)
        time.sleep(2) #must delay at least 1 second
        WebDriverWait(browser, 30).until(self.is_ready)
        # 设备类型 ，点击查询 按钮
        query_button_label_element = browser.find_element(By.XPATH, "//span[contains(text(), '查询')]")
        query_button_element = query_button_label_element.find_element(By.XPATH, "..") # button is span's parent
        query_button_element.click()
        time.sleep(2)
        WebDriverWait(browser, 30).until(self.is_ready)
        # exract total record number
        total_record_element = browser.find_element(By.XPATH, "//li[@class='ant-pagination-total-text']")
        if self.verbose:
            print(f'total_record_element: {total_record_element.text}')
        total_record_string = total_record_element.text
        match = re.search(r'共(\d+)', total_record_string)
        if match:
            if self.verbose:
                print(f'total_record_element: {int(match.group(1))}')
            total_record_number = int(match.group(1))
        browser.close()
        return total_record_number

    def scrape(self):
        pass

    def scrape_onepage(self, page):
        # option, todo
        ret = 0
        options = webdriver.ChromeOptions()
        ret = self.init_browser(options)
        if ret != 0:
            print(f"page {page}: fail to init_browser")
            return ret
        for i in range(10):
            ret = self.check_captcha()
            if ret != 0:
                print(f"page {page}: fail to check_captcha")
                if i < 9:
                    self.browser.refresh()
                    continue
                else:
                    return ret
            else:
                break
        ret = self.expand_query_button()
        if ret != 0:
            print(f"page {page}: fail to expand_query_button")
            return ret
        ret = self.jump_to_page(page)
        if ret != 0:
            print(f"page {page}: fail to jump_to_page")
            return ret
        ret = self.extract_data_to_file()
        if ret != 0:
            print(f"page {page}: fail to extract_data_to_file")
        self.uninit_browser()

    def init_browser(self, options=webdriver.ChromeOptions()):
        if self.verbose:
            pass
        else:
            options.add_argument("headless")
             
        if self.browser is None:
            browser = webdriver.Chrome(options=options)
            self.browser = browser
        if self.browser is not None:
            self.browser.get(self.url)
            WebDriverWait(self.browser, 30).until(self.is_ready)
            return 0
        else:
            return -1   # fail to init browser

    def uninit_browser(self):
        self.browser.close()
        self.browser = None
        return 0
    
    def check_captcha(self, option=True):
        if option == False:
            return 0
        # make sure captcha solved correctly
        if self.browser is None:
            print("browser not init")
            return -1
        self.browser.get(self.url)
        WebDriverWait(self.browser, 30).until(self.is_ready)
        #print(browser.page_source)
        #get captcha image verify-left-bar， verify-move-block
        try:
            div_element_moveblock = self.browser.find_element(By.CLASS_NAME, "verify-sub-block")
        except NoSuchElementException:
            if self.verbose:
                print("exception in check_captcha")
            return -1
        except Exception as e:
            print(f'{e} raised in check_captcha')
            return -1
        actions = ActionChains(self.browser)
        actions.move_to_element(div_element_moveblock).perform()
        actions.release()
        time.sleep(2)
        try:
            # Find the div element with class "verify-img-panel"
            div_element_bgimg =  self.browser.find_element(By.CLASS_NAME, "verify-img-panel")
            #div_element = browser.find_element(By.XPATH, "//div[@class='verify-img-panel']")
            # Find the image element inside the div element
            image_element_bg = div_element_bgimg.find_element(By.TAG_NAME, "img")
        except NoSuchElementException:
            print("exception in check_captcha")
            return -1
        except Exception as e:
            print(f'{e} raised in check_captcha')
            return -1
        # Get the base64 encoded image data from the "src" attribute
        image_data_bg = image_element_bg.get_attribute("src")[22:]
        # Decode the base64 data to obtain the image binary data
        try:
            image_binary_bg = base64.b64decode(image_data_bg)
            # Save the image data to a file
            with open(f"dynamic_background.png", "wb") as f:
                f.write(image_binary_bg)
        except base64.binascii.Error as e:
            if self.verbose:
                print("An error occurred while decoding background data:", str(e))
            return -1 # fail to extract background image
        try:
            #find move block
            div_element_blockimg =  self.browser.find_element(By.CLASS_NAME, "verify-move-block")
            # Find the image element inside the div element
            image_element_block = div_element_blockimg.find_element(By.TAG_NAME, "img")
            # Get the base64 encoded image data from the "src" attribute
            image_data_block = image_element_block.get_attribute("src")[22:]
            # Decode the base64 data to obtain the image binary data
        except NoSuchElementException:
            print("exception in check_captcha")
            return -1
        except Exception as e:
            print(f'{e} raised in check_captcha')
            return -1
        try:
            image_binary_block = base64.b64decode(image_data_block)
            # Save the image data to a file
            with open(f"dynamic_block.png", "wb") as f:
                f.write(image_binary_block)
        except base64.binascii.Error as e:
            if self.verbose:
                print("An error occurred while decoding slider data:", str(e))
            return -1 # fail to extract block image
        # continue to handle capthcha
        if os.path.isfile("dynamic_background.png") and os.path.isfile("dynamic_block.png"):
            pass
        else:
            print("no captcha image data, please try again...")
            return -1
            
        distance = self.captcha_handler.calculate_distance()
        if self.verbose:
            print(f'distance: {distance}')
        if distance == 0:
            if self.verbose:
                print("distance is 0, might need to refresh capthca")
            return -1
        distance = int(distance*1.277)
        actions.click_and_hold(div_element_blockimg).move_by_offset(distance+1, 0).release().perform()
        time.sleep(2)
        WebDriverWait(self.browser, 30).until(self.is_ready)
        time.sleep(2)
        return 0

    def expand_query_button(self):
        if self.browser is None:
            print("browser not init")
            return -1
        if self.verbose:
            print("find query and click")
        # find 展开 按钮
        try:
            #self.browser.refresh()
            expand_button_element = self.browser.find_element(By.XPATH, "//a[contains(text(), '展开')]")
            expand_button_element.click()
            #self.browser.execute_script("arguments[0].click", expand_button_element)
            time.sleep(2)
            WebDriverWait(self.browser, 30).until(self.is_ready)
            #select device type element
            device_type_label = self.browser.find_element(By.XPATH, "//label[text()='设备类别名称']")
            parent_of_device_type_label = device_type_label.find_element(By.XPATH, "..")
            grandparent_of_device_type_label = parent_of_device_type_label.find_element(By.XPATH, "..")
            device_type_combobox0 = grandparent_of_device_type_label.find_element(By.XPATH, ".//div[@class='ant-col ant-form-item-control-wrapper']") 
            #//////div[@class='ant-select ant-select-enabled']")
            device_type_combobox1 = device_type_combobox0.find_element(By.XPATH, ".//div[@class='ant-form-item-control']") 
            device_type_combobox2 = device_type_combobox1.find_element(By.XPATH, ".//span[@class='ant-form-item-children']") 
            device_type_combobox3 = device_type_combobox2.find_element(By.XPATH, ".//div[@class='ant-select ant-select-enabled']")
            device_type_combobox3.click()
            time.sleep(2)
            WebDriverWait(self.browser, 30).until(self.is_ready)
            device_type_combobox3.send_keys(Keys.ARROW_DOWN)
            device_type_combobox3.send_keys(Keys.ENTER)
            time.sleep(2) #must delay at least 1 second
            WebDriverWait(self.browser, 30).until(self.is_ready)
            # 设备类型 ，点击查询 按钮
            query_button_label_element = self.browser.find_element(By.XPATH, "//span[contains(text(), '查询')]")
            query_button_element = query_button_label_element.find_element(By.XPATH, "..") # button is span's parent
            query_button_element.click()
            time.sleep(2)
            WebDriverWait(self.browser, 30).until(self.is_ready)
            # exract total record number
            total_record_number = 0
            total_record_element = self.browser.find_element(By.XPATH, "//li[@class='ant-pagination-total-text']")
            if self.verbose:
                print(f'total_record_element: {total_record_element.text}')
            total_record_string = total_record_element.text
            match = re.search(r'共(\d+)', total_record_string)
            if match:
                if self.verbose:
                    print(f'total_record_element: {int(match.group(1))}')
                total_record_number = int(match.group(1))
            if total_record_number == 0:
                print(f'failure: <total record: {total_record_number} >')
                return -2 # check record, error if no record?
            return 0
        except NoSuchElementException:
            print("exception in expand_query_button")
            return -3
        except Exception as e:
            print(f'{e} rasied in expand_query_button')
            return -4

    def jump_to_page(self, page):
        if self.browser is None:
            print("browser not init")
            return -1
        try:
            #找到page，跳转到特定page，获取数据
            parent_of_input_page_element = self.browser.find_element(By.XPATH, "//div[@class='ant-pagination-options-quick-jumper']")
            input_page_element = parent_of_input_page_element.find_element(By.XPATH, ".//input[@type='text']")
            input_page_element.clear()
            time.sleep(1)
            input_page_element.send_keys(int(page))
            time.sleep(2)
            input_page_element.send_keys(Keys.ENTER)
            time.sleep(1)
            print(f'jump to page {int(page)}...')
            WebDriverWait(self.browser, 30).until(self.is_ready)
            # f = open(self.output_file, "a")
            # # # print data
            # table_titles = self.browser.find_elements(By.XPATH, "//span[@class='ant-table-column-title']")
            # title_list = []
            # for title in table_titles:
            #     title_list.append(title.text)
            # table_title = " ".join(title_list)
            # if self.verbose:
            #     print(table_title)
            # f.write(table_title)
            # elements = self.browser.find_elements(By.XPATH, "//tbody[@class='ant-table-tbody']")
            # for elem in elements:
            #     if self.verbose:
            #         print(elem.text)
            #     f.write(elem.text)
            # f.close()
            return 0
        except NoSuchElementException:
            print("exception in jump_to_page")
            return -2
        except Exception as e:
            print(f'{e} rasied in jump_to_page')
            return -3

    def extract_data_to_file(self):
        if self.browser is None:
            print("browser not init")
            return -1
        try:
            f = open(self.output_file, "a")
            # #table title
            table_titles = self.browser.find_elements(By.XPATH, "//span[@class='ant-table-column-title']")
            title_list = []
            for title in table_titles:
                title_list.append(title.text)
            table_title = " ".join(title_list)
            if self.verbose:
                print(table_title)
            f.write(table_title + '\n')
            elements = self.browser.find_elements(By.XPATH, "//tbody[@class='ant-table-tbody']")
            for elem in elements:
                if self.verbose:
                    print(elem.text)
                f.write(elem.text)
            f.close()
            return 0
        except NoSuchElementException:
            print("exception in extract_data_to_file")
            return -2
        except Exception as e:
            print(f'{e} raised in extract_data_to_file')
            return -3
        

if __name__ == "__main__":
    # # Example usage:
    parser = argparse.ArgumentParser(description='Grab one page each time according to argument passed')
    parser.add_argument('-b', '--beginpage', help='begin page number specified', required=True)
    parser.add_argument('-e', '--endpage', help='end page number specified', required=True)
    parser.add_argument('-f', '--outputfile', help='output file to save grabbed data', required=True)
    parser.add_argument('-v', '--verbose', help='Increase output verbosity', action='store_true')
    args = parser.parse_args()
    begin_page = int(args.beginpage)
    end_page = int(args.endpage)
    output_file = args.outputfile
    output_file = output_file + '-' + f'{begin_page}' + '-to-' + f'{end_page}' + '.csv'
    verbose = False
    if args.verbose:
        print('Verbose output enabled')
        verbose = True
    print(f"handle page number: {begin_page} to {end_page}")
    scraper = TJWebScraper(
        "http://60.29.202.86:8090/#/ViewBsdtJxbaList", 
        begin_page, end_page, output_file, verbose)
    # for i in range(10):
    #     print(f' try {i} times...')
    #     ret = scraper.fetch_total_record()
    #     if ret == -1:
    #         print("fail to fetch total record")
    #         continue
    #     else:
    #         print(f'loop {i}, ret: {ret}')
    #         break
    for page_number in range(begin_page, end_page+1):
        for i in range(10):
            print(f' try {i} times...')
            ret = scraper.scrape_onepage(page_number)
            if ret == -1:
                print(f"fail to fetch page({page_number})")
                continue
            else:
                print(f'loop {i},page {page_number} ret: {ret}')
                break

